package com.contextspace;

import static org.junit.Assert.*;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FetchType;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToOne;
import javax.persistence.OneToMany;
import javax.persistence.Persistence;
import javax.persistence.TableGenerator;
import javax.persistence.TypedQuery;

import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.orm.jpa.JpaTransactionManager;
import org.springframework.transaction.TransactionStatus;
import org.springframework.transaction.support.DefaultTransactionDefinition;

public class TestOneToMany {

    static Logger log = LoggerFactory.getLogger(TestOneToMany.class.getName());
    
    @Test
    public void test() {
        
        EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb:$objectdb/db/test.tmp;drop");
        EntityManager em = emf.createEntityManager();
        JpaTransactionManager tm = new JpaTransactionManager(emf);
        
        Team teamMovieStars = new Team();
        teamMovieStars.setName("Team Movie Stars");
        //teamMovieStars.addPlayer(new Player("Brad Pitt"));
        //teamMovieStars.addPlayer(new Player("Angelina Jolie"));
        
        em.getTransaction().begin();
        em.persist(teamMovieStars);
        em.getTransaction().commit();
        em.flush();
        
        teamMovieStars.addPlayer(new Player("Brad Pitt"));
        teamMovieStars.addPlayer(new Player("Angelina Jolie"));
        
        em.getTransaction().begin();
        em.merge(teamMovieStars);
        em.getTransaction().commit();
        
        TypedQuery<Long> query = em.createQuery("SELECT count(c) FROM Player c", Long.class);
        Long cdiCount = query.getSingleResult();
        assertEquals(2, cdiCount.intValue());
        
        TypedQuery<Player> query2 = em.createQuery("SELECT c FROM Player c where c.name='Brad Pitt'", Player.class);
        List<Player> results = query2.getResultList();
        assertEquals(1, results.size());
        assertEquals("Brad Pitt", results.get(0).getName());
        assertNotNull(results.get(0).getTeam());
        assertEquals("Team Movie Stars", results.get(0).getTeam().getName());
        
        
        TypedQuery<Team> query3 = em.createQuery("SELECT c FROM Team c where c.name='Team Movie Stars'", Team.class);
        List<Team> teams = query3.getResultList();
        assertEquals(1, teams.size());
        
        teamMovieStars = teams.get(0);
        teamMovieStars.removePlayer("Brad Pitt");
        
        em.getTransaction().begin();
        em.merge(teamMovieStars);
        em.getTransaction().commit();
        em.flush();
        
        TypedQuery<Long> query4 = em.createQuery("SELECT count(c) FROM Player c", Long.class);
        Long cdiCount2 = query4.getSingleResult();
        assertEquals(1, cdiCount2.intValue());
        
        TypedQuery<Team> query5 = em.createQuery("SELECT c FROM Team c", Team.class);
        List<Team> teams2 = query5.getResultList();
        assertEquals(1, teams2.size());
        assertEquals(1, teams2.get(0).getSquad().size());
        
        em.getTransaction().begin();
        teamMovieStars = teams2.get(0);
        teamMovieStars.addPlayer(new Player("Mark Wahlberg"));
        teamMovieStars.addPlayer(new Player("Charlize Theron"));
        em.merge(teamMovieStars);
        em.getTransaction().commit();
        em.flush();
        
        TypedQuery<Team> query6 = em.createQuery("SELECT c FROM Team c", Team.class);
        List<Team> teams3 = query6.getResultList();
        assertEquals(1, teams3.size());
        assertEquals(3, teams3.get(0).getSquad().size());
        
        em.close();
        emf.close();
    }

    @Entity
    @TableGenerator(name="IdTableGen", initialValue=1, allocationSize=50)
    public static class Team implements Serializable {

        private static final long serialVersionUID = 1L;
        private long id;
        private List<Player> squad = new ArrayList<Player>();
        private String name = null;

        /**
         * @return the object id
         */
        @Id
        @GeneratedValue(strategy=GenerationType.TABLE, generator="IdTableGen")
        public long getId() {
            return id;
        }

        /**
         * @param id the id to be set
         */
        public void setId(long id) {
            this.id = id;
        }
        
        @OneToMany(targetEntity = Player.class, mappedBy = "team", cascade = CascadeType.ALL, fetch = FetchType.EAGER, orphanRemoval=true)
        public List<Player> getSquad() {
            return squad;
        }

        /**
         * @param squad the squad to set
         */
        public void setSquad(List<Player> squad) {
            this.squad = squad;
        }
        
        public void addPlayer(Player player) {
            player.team = this;
            this.squad.add(player);
        }
        
        public void removePlayer(String name) {
            Player playerToRemove = null;
            for(Player player : squad) {
                if(player.getName().equals(name)) {
                    playerToRemove = player;
                }
            }
            if(playerToRemove != null) {
                squad.remove(playerToRemove);
            }
            System.out.println("Squad size: " + squad.size());
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }
    }

    @Entity
    public static class Player implements Serializable {

        private static final long serialVersionUID = 1L;
 
        private long id;
        private Team team;
        private String name = null;

        /**
         * Default Constructor
         */
        public Player() {
            
        }
        
        /**
         * Constructor
         * @param name
         */
        public Player(String name) {
            this.name = name;
        }

        /**
         * @return the object id
         */
        @Id
        @GeneratedValue(strategy=GenerationType.TABLE, generator="IdTableGen")
        public long getId() {
            return id;
        }

        /**
         * @param id the id to be set
         */
        public void setId(long id) {
            this.id = id;
        }
        
        @ManyToOne
        public Team getTeam() {
            return team;
        }
        
        public void setTeam(Team team) {
            this.team = team;
        }

        /**
         * @return the name
         */
        public String getName() {
            return name;
        }

        /**
         * @param name the name to set
         */
        public void setName(String name) {
            this.name = name;
        }  
    }

}
